package org.wbconfig;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.Collection;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferencePage;
import org.eclipse.jface.preference.IPreferencePageContainer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.JFacePreferences;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.wbconfig.action.ActionMap;
import org.wbconfig.action.BaseAction;
import org.wbconfig.action.file.RevertAction;
import org.wbconfig.action.file.SaveAction;
import org.wbconfig.action.file.SaveAsAction;
import org.wbconfig.preference.AppPreferenceStore;
import org.wbconfig.preference.ConfigPreferenceStore;
import org.wbconfig.resource.ResourceManager;
import org.wbconfig.util.ExceptionHandler;
import org.wbconfig.widget.AboutTab;
import org.wbconfig.widget.DisplayTab;
import org.wbconfig.widget.FlightTab;
import org.wbconfig.widget.GeneralTab;
import org.wbconfig.widget.PreloadTab;

/**
 * WBConfig.java
 *
 * @author Xavier Cho
 * @version $Revision: 1.10 $ $Date: 2004/01/24 14:25:14 $
 */
public class WBConfig
    extends ApplicationWindow
    implements IPreferencePageContainer {
    private static Log log = LogFactory.getLog(WBConfig.class);

    private ActionMap actionMap;
    private TabFolder tabFolder;
    private IPreferencePage[] preferencePages;

    /**
     * Defualt constructor of WBConfig class
     */
    public WBConfig() {
        super(null);
    }

    /**
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(final Shell shell) {
        if (log.isDebugEnabled()) {
            log.debug("Configuring shell...");
        }

        this.actionMap = createActionMap();

        ResourceManager resources = ResourceManager.getInstance();

        if (log.isDebugEnabled()) {
            log.debug("Initializing preference store...");
        }

        try {
            IPreferenceStore preferences = new AppPreferenceStore();
            JFacePreferences.setPreferenceStore(preferences);
        } catch (IOException e) {
            String msg = resources.getString("error.message.preferences");
            ExceptionHandler.handleException(msg, e);

            System.exit(-1);
        }

        setBlockOnOpen(true);
        addMenuBar();
        addToolBar(SWT.FLAT);
        addStatusLine();

        super.configureShell(shell);

        Window.setExceptionHandler(new ExceptionHandler());

        shell.setText(resources.getString("application.title"));

        Image icon = resources.getImage("application.icon");
        if (icon != null) {
            shell.setImage(icon);
        }

        Point size = getInitialSize();
        Point location = getInitialLocation(size);

        shell.setBounds(location.x, location.y, size.x, size.y);
    }

    protected ActionMap createActionMap() {
        if (log.isDebugEnabled()) {
            log.debug("Initializing action map...");
        }

        ActionMap map = new ActionMap();

        ResourceManager resources = ResourceManager.getInstance();

        Collection actions = resources.getActionNames();
        Iterator it = actions.iterator();

        while (it.hasNext()) {
            String name = (String) it.next();

            try {
                Constructor constructor =
                    Class.forName(name).getConstructors()[0];

                Object[] args = new Object[] {this};

                BaseAction action = (BaseAction) constructor.newInstance(args);

                map.put(action);
            } catch (Exception e) {
                log.warn("Unable to instantiate action class " + name, e);
            }
        }

        if (log.isDebugEnabled()) {
            log.debug("Loaded " + actions.size() + " action classes.");
        }

        return map;
    }

    /**
     * @see org.eclipse.jface.window.ApplicationWindow#createMenuManager()
     */
    protected MenuManager createMenuManager() {
        return new AppMenuManager(this);
    }

    /**
     * @see org.eclipse.jface.window.ApplicationWindow#createToolBarManager(int)
     */
    protected ToolBarManager createToolBarManager(final int style) {
        return new AppToolBarManager(this, style);
    }

    /**
     * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
     */
    protected Control createContents(final Composite parent) {
        Composite contents = new Composite(parent, SWT.NULL);
        contents.setLayout(new FillLayout());

        this.tabFolder = new TabFolder(contents, SWT.BORDER);

        ResourceManager resources = ResourceManager.getInstance();

        this.preferencePages =
            new IPreferencePage[] {
                new AboutTab(resources.getString("tab.about")),
                new DisplayTab(resources.getString("tab.display")),
                new FlightTab(resources.getString("tab.flight")),
                new GeneralTab(resources.getString("tab.general")),
                new PreloadTab(resources.getString("tab.preload"))};

        int size = preferencePages.length;

        for (int i = 0; i < size; i++) {
            IPreferencePage page = preferencePages[i];
            page.createControl(tabFolder);
            page.setContainer(this);

            TabItem item = new TabItem(tabFolder, SWT.NULL);
            item.setText(page.getTitle());
            item.setControl(page.getControl());
        }

        updateButtons();

        return contents;
    }

    /**
     * @see org.eclipse.jface.window.Window#getInitialLocation(org.eclipse.swt.graphics.Point)
     */
    protected Point getInitialLocation(final Point initialSize) {
        IPreferenceStore preferences = JFacePreferences.getPreferenceStore();

        int x = preferences.getInt("location.x");
        int y = preferences.getInt("location.y");

        return new Point(x, y);
    }

    /**
     * @see org.eclipse.jface.window.Window#getInitialSize()
     */
    protected Point getInitialSize() {
        IPreferenceStore preferences = JFacePreferences.getPreferenceStore();

        int x = preferences.getInt("size.width");
        int y = preferences.getInt("size.height");

        return new Point(x, y);
    }

    protected ActionMap getActionMap() {
        return actionMap;
    }

    /**
     * @return
     */
    public IPreferencePage[] getPreferencePages() {
        return preferencePages;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferencePageContainer#getPreferenceStore()
     */
    public IPreferenceStore getPreferenceStore() {
        IPreferenceStore preferences = ConfigPreferenceStore.getInstance();
        return preferences;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferencePageContainer#updateButtons()
     */
    public void updateButtons() {
        ConfigPreferenceStore preferences = ConfigPreferenceStore.getInstance();

        boolean isDirty = preferences.needsSaving();
        boolean isValid = true;
        boolean hasFileOpened = (preferences.getFilename() != null);

        int size = preferencePages.length;

        for (int i = 0; i < size; i++) {
            IPreferencePage page = preferencePages[i];
            if (!page.isValid()) {
                isValid = false;
                break;
            }
        }

        ActionMap map = getActionMap();
        map.get(SaveAction.class).setEnabled(
            hasFileOpened && isDirty && isValid);
        map.get(RevertAction.class).setEnabled(
            hasFileOpened && isDirty && isValid);
        map.get(SaveAsAction.class).setEnabled(isValid);
    }

    /**
     * @see org.eclipse.jface.preference.IPreferencePageContainer#updateMessage()
     */
    public void updateMessage() {
        int index = tabFolder.getSelectionIndex();

        ResourceManager resources = ResourceManager.getInstance();
        Image icon = resources.getImage("image.error");
        String message = preferencePages[index].getErrorMessage();

        IStatusLineManager manager = getStatusLineManager();

        if (message == null) {
            manager.setErrorMessage(null);
        } else {
            manager.setErrorMessage(icon, message);
        }
    }

    /**
     * @see org.eclipse.jface.preference.IPreferencePageContainer#updateTitle()
     */
    public void updateTitle() {
    }

    /**
     * @see org.eclipse.jface.window.Window#canHandleShellCloseEvent()
     */
    protected boolean canHandleShellCloseEvent() {
        return true;
    }

    /**
     * @see org.eclipse.jface.window.Window#handleShellCloseEvent()
     */
    protected void handleShellCloseEvent() {
        boolean isDirty = ConfigPreferenceStore.getInstance().needsSaving();

        if (isDirty) {
            ResourceManager resources = ResourceManager.getInstance();
            String title = resources.getString("dialog.title.confirm");
            String message =
                resources.getString("message.exit.discard.changes");

            if (!MessageDialog.openConfirm(getShell(), title, message)) {
                return;
            }
        }

        Shell shell = getShell();

        Point location = shell.getLocation();
        Point size = shell.getSize();

        super.handleShellCloseEvent();

        if (log.isInfoEnabled()) {
            log.info("Saving user preferences...");
        }

        AppPreferenceStore preferences =
            (AppPreferenceStore) JFacePreferences.getPreferenceStore();

        preferences.setValue("location.x", location.x);
        preferences.setValue("location.y", location.y);
        preferences.setValue("size.width", size.x);
        preferences.setValue("size.height", size.y);

        try {
            preferences.save();
        } catch (IOException e) {
            if (log.isWarnEnabled()) {
                String msg = "Failed to save user preferences data.";
                log.warn(msg, e);
            }
        }
    }

    public static void main(final String[] args) {
        try {
            ApplicationWindow application = new WBConfig();
            application.open();
        } catch (Throwable t) {
            if (log.isFatalEnabled()) {
                String msg = "Unable to initialize the application.";
                log.fatal(msg, t);
            }
        } finally {
            Display.getCurrent().dispose();
            System.exit(0);
        }
    }
}
