package org.wbconfig.preference;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.lang.NullArgumentException;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.jface.preference.PreferenceStore;
import org.wbconfig.util.ConfigLineParser;

/**
 * ConfigPreferenceStore.java
 *
 * @author Xavier Cho
 * @version $Revision: 1.9 $ $Date: 2004/01/31 06:07:53 $
 */
public final class ConfigPreferenceStore extends PreferenceStore {
    private static ConfigPreferenceStore instance;
    private static Log log = LogFactory.getLog(ConfigPreferenceStore.class);

    private String filename;
    private boolean dirty = false;

    private ConfigPreferenceStore() {
        setDefault("radiowidth", 400);
        setDefault("radiolines", 4);
    }

    public static ConfigPreferenceStore getInstance() {
        if (instance == null) {
            synchronized (ConfigPreferenceStore.class) {
                instance = new ConfigPreferenceStore();
            }
        }

        return instance;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferenceStore#needsSaving()
     */
    public boolean needsSaving() {
        return dirty;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferenceStore#putValue(java.lang.String,
     *             java.lang.String)
     */
    public void putValue(final String key, final String value) {
        super.putValue(key, value);
        this.dirty = true;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferenceStore#setToDefault(java.lang.String)
     */
    public void setToDefault(final String key) {
        super.setToDefault(key);
        this.dirty = true;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
     *             boolean)
     */
    public void setValue(final String key, final boolean value) {
        super.setValue(key, value);
        this.dirty = true;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
     *             double)
     */
    public void setValue(final String key, final double value) {
        super.setValue(key, value);
        this.dirty = true;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
     *             float)
     */
    public void setValue(final String key, final float value) {
        super.setValue(key, value);
        this.dirty = true;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
     *             int)
     */
    public void setValue(final String key, final int value) {
        super.setValue(key, value);
        this.dirty = true;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
     *             long)
     */
    public void setValue(final String key, final long value) {
        super.setValue(key, value);
        this.dirty = true;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferenceStore#setValue(java.lang.String,
     *             java.lang.String)
     */
    public void setValue(final String key, final String value) {
        super.setValue(key, value);
        this.dirty = true;
    }

    public String getFilename() {
        return filename;
    }

    /**
     * @see org.eclipse.jface.preference.PreferenceStore#setFilename(java.lang.String)
     */
    public void setFilename(final String string) {
        super.setFilename(string);

        this.filename = string;
    }

    /**
     * @see org.eclipse.jface.preference.PreferenceStore#load(java.io.InputStream)
     */
    public void load(final InputStream in) throws IOException {
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        String line = null;

        while ((line = reader.readLine()) != null) {
            line = line.toLowerCase().trim();

            if (line.startsWith(".")) {
                String key = line.substring(1);

                if (line.indexOf(" ") == -1) {
                    setValue(line + ".toggle", true);
                }
            }
        }

        this.dirty = false;
    }

    /**
     * @see org.eclipse.jface.preference.PreferenceStore#save(java.io.OutputStream,
     *             java.lang.String)
     */
    public void save(final OutputStream out, final String headers)
        throws IOException {

        if (log.isDebugEnabled()) {
            log.debug("Saving configuration settings : ");
            log.debug(" - file : " + filename);
        }

        PrintWriter writer = new PrintWriter(new OutputStreamWriter(out));

        if (!StringUtils.isEmpty(headers)) {
            writer.println(headers);
        }

        list(writer);

        this.dirty = false;
    }

    /**
     * @see org.eclipse.jface.preference.PreferenceStore#load()
     */
    public void load() throws IOException {
        if (filename == null) {
            String msg = "No input file has been specified.";
            throw new IOException(msg);
        }

        if (log.isDebugEnabled()) {
            log.debug("Loading configuration settings : ");
            log.debug(" - file : " + filename);
        }

        File file = new File(filename);

        BufferedReader reader = new BufferedReader(new FileReader(file));
        try {
            ConfigLineParser parser = new ConfigLineParser();
            String line = null;
            while ((line = reader.readLine()) != null) {
                parser.add(line);
            }

            Map properties = parser.getPropertiesMap();

            Iterator it = properties.keySet().iterator();
            while (it.hasNext()) {
                Object key = it.next();
                Object value = properties.get(key);

                setValue(key.toString(), value.toString());
            }

            this.dirty = false;
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    if (log.isWarnEnabled()) {
                        String msg =
                            "Unable to close file reader : " + e.getMessage();
                        log.warn(msg, e);
                    }
                }
            }
        }
    }

    /**
     * @see org.eclipse.jface.preference.PreferenceStore#list(java.io.PrintWriter)
     */
    public void list(final PrintWriter writer) {
        if (writer == null) {
            throw new NullArgumentException("writer");
        }

        String[] names = preferenceNames();
        if (names != null) {
            Arrays.sort(names);

            int size = names.length;
            for (int i = 0; i < size; i++) {
                String key = names[i].toLowerCase();

                if (log.isTraceEnabled()) {
                    log.trace(" - " + key + " = " + getString(key));
                }

                int index = key.lastIndexOf(".");
                if (index == -1) {
                    writer.print(".");
                    writer.print(key);
                    writer.print(" ");
                    writer.print(getString(key));
                } else {
                    String prefix = key.substring(0, index);
                    String suffix = key.substring(index + 1);

                    if (suffix.equals("toggle")) {
                        if (getBoolean(key)) {
                            writer.print(".");
                            writer.print(prefix);
                        }
                    } else if (suffix.equals("boolean")) {
                        writer.print(".");
                        writer.print(prefix);

                        if (getBoolean(key)) {
                            writer.print(" 1");
                        } else {
                            writer.print(" 0");
                        }
                    } else if (StringUtils.isNumeric(suffix)) {
                        if (getBoolean(key)) {
                            writer.print(".");
                            writer.print(prefix);
                            writer.print(" ");
                            writer.print(suffix);
                        }
                    }
                }

                writer.println();
            }

            writer.flush();
        }
    }
}
