package org.wbconfig.resource;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;

/**
 * ResourceManager.java
 * @author Xavier Cho
 * @version $Revision: 1.1 $ $Date: 2003/12/31 09:29:54 $
 */
public final class ResourceManager {
    public static final String RESOURCE_BUNDLE =
        "org.wbconfig.resource.ApplicationResources";
    public static final String ACTION_PREFIX = "org.wbconfig.action";

    private static ResourceManager instance;
    private ImageRegistry imageRegistry;
    private ResourceBundle bundle;

    private ResourceManager() {
        this.bundle = ResourceBundle.getBundle(RESOURCE_BUNDLE);
    }

    public static ResourceManager getInstance() {
        if (instance == null) {
            synchronized (ResourceManager.class) {
                instance = new ResourceManager();
            }
        }
        return instance;
    }

    public String getString(final String key) {
        if (key == null) {
            SWT.error(SWT.ERROR_NULL_ARGUMENT);
        }

        String value;

        try {
            value = bundle.getString(key);
        } catch (MissingResourceException e) {
            value = null;
        }

        return value;
    }

    public String getString(final String key, final Object[] args) {
        if (key == null) {
            SWT.error(SWT.ERROR_NULL_ARGUMENT);
        }

        String value;

        try {
            value = bundle.getString(key);
            value = MessageFormat.format(value, args);
        } catch (MissingResourceException e) {
            value = null;
        }

        return value;
    }

    public Image getImage(final String key) {
        if (imageRegistry == null) {
            synchronized (this) {
                this.imageRegistry = new ImageRegistry();
            }
        }

        Image image = imageRegistry.get(key);

        if (image == null) {
            String path = getString(key);

            if (path != null) {
                ImageDescriptor descriptor =
                    ImageDescriptor.createFromFile(ResourceManager.class, path);
                imageRegistry.put(key, descriptor);
                image = imageRegistry.get(key);
            }
        }

        return image;
    }

    public ImageDescriptor getImageDescriptor(final String key) {
        if (imageRegistry == null) {
            synchronized (this) {
                this.imageRegistry = new ImageRegistry();
            }
        }

        ImageDescriptor descriptor = imageRegistry.getDescriptor(key);

        if (descriptor == null) {
            String path = getString(key);

            if (path != null) {
                descriptor =
                    ImageDescriptor.createFromFile(ResourceManager.class, path);
                imageRegistry.put(key, descriptor);
            }
        }

        return descriptor;
    }

    public Collection getActionNames() {
        Collection actions = new ArrayList(20);

        Enumeration enum = bundle.getKeys();
        while (enum.hasMoreElements()) {
            String name = (String) enum.nextElement();

            if (name.startsWith(ACTION_PREFIX)) {
                int index = name.lastIndexOf("Action") + 6;
                name = name.substring(0, index);

                if (!actions.contains(name)) {
                    actions.add(name);
                }
            }
        }

        return actions;
    }
}